@@ -82,6 +82,10 @@ def set_api_base_url(api_base_url: str) -> None:
     )
 
 
+def set_base_url(base_url: str) -> None:
+    return set_api_base_url(base_url)
+
+
 def set_api_token(api_token: str) -> None:
     api_client = _BioLibApiClient.get()
     api_client.sign_in_with_api_token(api_token)
