@@ -16,18 +16,20 @@ class JobState(Enum):
 
 class _Job(TypedDict):
     app_version: AppVersionOnJob
+    arguments_override_command: bool
     auth_token: str
     caller_job: Optional[str]
     created_at: str
     federated_job_uuid: Optional[str]
     public_id: str
+    uuid: str
     remote_hosts_with_warning: List[RemoteHost]
+    state: str
     user_id: Optional[str]
-    arguments_override_command: bool
 
 
 # type optional keys with total=False
-class Job(_Job, total=False):
+class CreatedJobDict(_Job, total=False):
     custom_compute_node_url: str
 
 
@@ -42,6 +44,6 @@ class JobWrapper(TypedDict):
     access_token: str
     BASE_URL: str  # TODO: refactor this to lower case
     compute_node_info: Optional[ComputeNodeInfo]
-    job: Job
+    job: CreatedJobDict
     cloud_job: Optional[CloudJob]
     job_temporary_dir: Optional[str]
