@@ -1,10 +1,18 @@
 from abc import ABC, abstractmethod
-
 import io
-import requests
+import time
 
-from biolib.typing_utils import Iterable
-from biolib.utils.multipart_uploader import get_chunk_iterator_from_bytes
+import requests
+from requests import Response
+
+from biolib.biolib_logging import logger
+from biolib.typing_utils import Optional, Dict
+
+
+class RemoteEndpoint(ABC):
+    @abstractmethod
+    def get_remote_url(self):
+        pass
 
 
 class IndexableBuffer(ABC):
@@ -16,10 +24,6 @@ class IndexableBuffer(ABC):
     def get_data(self, start: int, length: int) -> bytes:
         pass
 
-    @abstractmethod
-    def get_data_as_iterable(self, start: int, length: int, chunk_size: int) -> Iterable[bytes]:
-        pass
-
     def get_data_as_string(self, start: int, length: int) -> str:
         return self.get_data(start=start, length=length).decode()
 
@@ -42,11 +46,51 @@ class IndexableBuffer(ABC):
         return data
 
 
+def _get_request_with_retries_for_timeout(
+        url: str,
+        headers: Optional[Dict[str, str]] = None,
+        stream: bool = False,
+) -> Response:
+    retries = 10
+    for retry_count in range(retries):
+        if retry_count > 0:
+            time.sleep(1 * retry_count)
+            logger.debug('Retrying HTTP GET request...')
+
+        try:
+            response = requests.get(
+                headers=headers,
+                stream=stream,
+                timeout=10,
+                url=url,
+            )
+        except requests.exceptions.ReadTimeout:
+            logger.debug('Got read timeout of HTTP GET request')
+            continue
+
+        if response.status_code == 502:
+            logger.debug('Got 502 status for HTTP GET')
+            continue
+
+        if response.status_code == 400:
+            logger.error(
+                f'GET request for "{url}" got response with status {response.status_code} and '
+                f'content: {response.content.decode()}'
+            )
+
+        if not response.ok:
+            raise Exception(f'GET request for "{url}" got response with status {response.status_code}')
+
+        return response
+
+    raise Exception(f'Failed to complete HTTP GET request for "{url}" after {retries} retries')
+
+
 class RemoteIndexableBuffer(IndexableBuffer):
 
-    def __init__(self, url: str):
+    def __init__(self, endpoint: RemoteEndpoint):
         super().__init__()
-        self._url = url
+        self._endpoint = endpoint
 
     def get_data(self, start: int, length: int) -> bytes:
         if length < 0:
@@ -56,9 +100,10 @@ class RemoteIndexableBuffer(IndexableBuffer):
             return bytes(0)
 
         end = start + length - 1
-        response = requests.get(url=self._url, headers={'range': f'bytes={start}-{end}'})
-        if not response.ok:
-            raise Exception(f'get_data got not ok response status {response.status_code}')
+        response = _get_request_with_retries_for_timeout(
+            url=self._endpoint.get_remote_url(),
+            headers={'range': f'bytes={start}-{end}'},
+        )
 
         data: bytes = response.content
         if len(data) != length:
@@ -66,22 +111,6 @@ class RemoteIndexableBuffer(IndexableBuffer):
 
         return data
 
-    def get_data_as_iterable(self, start: int, length: int, chunk_size: int) -> Iterable[bytes]:
-        if length < 0:
-            raise Exception('get_data length must be positive')
-
-        if length == 0:
-            return []
-
-        end = start + length - 1
-        response = requests.get(
-            url=self._url,
-            headers={'range': f'bytes={start}-{end}'},
-            stream=True,
-            timeout=60,
-        )
-        return response.iter_content(chunk_size=chunk_size)
-
 
 class InMemoryIndexableBuffer(IndexableBuffer):
 
@@ -94,9 +123,6 @@ class InMemoryIndexableBuffer(IndexableBuffer):
         end = start + length
         return self._buffer[start:end]
 
-    def get_data_as_iterable(self, start: int, length: int, chunk_size: int) -> Iterable[bytes]:
-        return get_chunk_iterator_from_bytes(self._buffer[start:start + length], chunk_size)
-
     def __len__(self):
         return self._length_bytes
 
@@ -116,11 +142,16 @@ class LazyLoadedFile:
     def path(self) -> str:
         return self._path
 
+    @property
+    def start(self) -> int:
+        return self._start
+
+    @property
+    def length(self) -> int:
+        return self._length
+
     def get_file_handle(self) -> io.BufferedIOBase:
         return io.BytesIO(self.get_data())
 
     def get_data(self) -> bytes:
         return self._buffer.get_data(start=self._start, length=self._length)
-
-    def get_data_as_iterable(self) -> Iterable[bytes]:
-        return self._buffer.get_data_as_iterable(start=self._start, length=self._length, chunk_size=1_000_000)
