@@ -28,3 +28,15 @@ class StorageDownloadFailed(Exception):
 
 class CloudJobFinishedError(Exception):
     pass
+
+
+class JobResultError(BioLibError):
+    pass
+
+
+class JobResultNotFound(JobResultError):
+    pass
+
+
+class JobResultPermissionError(JobResultError):
+    pass
