@@ -91,24 +91,33 @@ class CloudUtils:
             return
 
         config = CloudUtils.get_webserver_config()
-        try:
-            requests.post(
-                url=f'{config["base_url"]}/api/jobs/cloud/finish/',
-                json={
-                    'auth_token': config["compute_node_info"]["auth_token"],
-                    'cloud_job_id': cloud_job_id,
-                    'system_exception_code': system_exception_code,
-                },
-                timeout=5,
-            )
 
-            opt_error_string = f' with error code {system_exception_code}' if system_exception_code else ''
-            logger_no_user_data.debug(
-                f'Cloud Job "{cloud_job_id}" was reported as finished' + opt_error_string
-            )
+        for _ in range(100):
+            try:
+                response = requests.post(
+                    url=f'{config["base_url"]}/api/jobs/cloud/finish/',
+                    json={
+                        'auth_token': config["compute_node_info"]["auth_token"],
+                        'cloud_job_id': cloud_job_id,
+                        'system_exception_code': system_exception_code,
+                    },
+                    timeout=10,
+                )
+
+                response.raise_for_status()
+
+                opt_error_string = f' with error code {system_exception_code}' if system_exception_code else ''
+                logger_no_user_data.debug(
+                    f'Cloud Job "{cloud_job_id}" was reported as finished' + opt_error_string
+                )
+                return
+
+            except Exception as error:  # pylint: disable=broad-except
+                logger_no_user_data.debug(
+                    f'Could not finish cloud job "{cloud_job_id}" due to {error}, retrying...'
+                )
+                time.sleep(10)
 
-        except Exception as error:  # pylint: disable=broad-except
-            logger_no_user_data.error(f'Could not finish cloud job with id: {cloud_job_id} got error: {error}')
 
     @staticmethod
     def _report_availability() -> None:
