@@ -1,3 +1,4 @@
+import json
 import os
 import shutil
 import uuid
@@ -5,7 +6,7 @@ import uuid
 from biolib.biolib_logging import logger_no_user_data
 from biolib.compute_node.job_worker.cache_types import LfsCacheStateDict, UuidStr, StoragePartition, \
     DockerImageCacheStateDict
-from biolib.typing_utils import Dict, List
+from biolib.typing_utils import Dict, List, cast, Optional
 from biolib.utils.cache_state import CacheState, CacheStateError
 
 
@@ -18,6 +19,7 @@ class LfsCacheState(CacheState):
     def __init__(self):
         super().__init__()
 
+        self._storage_path_for_write: str = self._get_storage_path_for_write()
         self._storage_paths: List[str] = self._get_storage_paths()
         self._tmp_storage_paths: List[str] = self._get_tmp_storage_paths()
 
@@ -25,6 +27,10 @@ class LfsCacheState(CacheState):
     def storage_paths(self) -> List[str]:
         return self._storage_paths
 
+    @property
+    def storage_path_for_write(self) -> str:
+        return self._storage_path_for_write
+
     @property
     def tmp_storage_paths(self) -> List[str]:
         return self._tmp_storage_paths
@@ -37,10 +43,19 @@ class LfsCacheState(CacheState):
     def _state_path(self) -> str:
         state_path = os.environ.get('BIOLIB_LFS_STATE_PATH')
         if not state_path:
-            raise Exception('Large File System state path not set')
+            raise CacheStateError('Environment variable "BIOLIB_LFS_STATE_PATH" not set')
 
         return state_path
 
+    def get_read_only_dict_without_lock(self) -> Optional[LfsCacheStateDict]:
+        try:
+            if not os.path.isfile(self._state_path):
+                return None
+            with open(self._state_path, mode='r') as file:
+                return cast(LfsCacheStateDict, json.load(file))
+        except BaseException:
+            return None
+
     def _get_default_state(self) -> LfsCacheStateDict:
         return LfsCacheStateDict(
             large_file_systems={},
@@ -78,6 +93,19 @@ class LfsCacheState(CacheState):
 
         return lfs_storage_paths_list
 
+    @staticmethod
+    def _get_storage_path_for_write() -> str:
+        lfs_storage_path_for_write = os.environ.get('BIOLIB_LFS_WRITE_STORAGE_PATH')
+        logger_no_user_data.debug(f'lfs_storage_path_for_write={lfs_storage_path_for_write}')
+
+        if not lfs_storage_path_for_write:
+            raise CacheStateError('Environment variable "BIOLIB_LFS_WRITE_STORAGE_PATH" not set')
+
+        if not os.path.isdir(lfs_storage_path_for_write):
+            raise CacheStateError(f'LFS storage path {lfs_storage_path_for_write} is not a directory')
+
+        return lfs_storage_path_for_write
+
     def _get_storage_partitions_from_env(self) -> Dict[UuidStr, StoragePartition]:
         storage_states: Dict[UuidStr, StoragePartition] = {}
         for lfs_storage_path in self._storage_paths:
@@ -96,7 +124,7 @@ class LfsCacheState(CacheState):
 class DockerImageCacheState(CacheState):
     @property
     def _state_path(self) -> str:
-        return f'{super()._cache_dir}/docker-cache-state.json'
+        return f'{self._user_cache_dir}/docker-cache-state.json'
 
     def _get_default_state(self) -> DockerImageCacheStateDict:
         return {}
