@@ -7,34 +7,37 @@ import os
 import io
 import re
 import shlex
+from copy import copy
 from datetime import datetime
 
 import docker  # type: ignore
 import docker.types  # type: ignore
 
-from docker.errors import ImageNotFound, APIError  # type: ignore
+from docker.errors import ImageNotFound  # type: ignore
 from docker.models.containers import Container  # type: ignore
 
 from biolib import utils
-from biolib.biolib_binary_format import ModuleOutput, ModuleInput
+from biolib.biolib_binary_format import ModuleInput, ModuleOutputV2
 from biolib.biolib_docker_client import BiolibDockerClient
 from biolib.biolib_errors import DockerContainerNotFoundDuringExecutionException, BioLibError
 from biolib.biolib_logging import logger, logger_no_user_data
 from biolib.compute_node import utils as compute_node_utils
 from biolib.compute_node.job_worker.docker_image_cache import DockerImageCache
-from biolib.compute_node.job_worker.executors.base_executor import BaseExecutor
+from biolib.biolib_binary_format.file_in_container import FileInContainer
+from biolib.compute_node.job_worker.executors.docker_types import DockerDiffKind
 from biolib.compute_node.job_worker.executors.types import StatusUpdate, LocalExecutorOptions
 from biolib.compute_node.job_worker.mappings import Mappings, path_without_first_folder
 from biolib.compute_node.job_worker.utils import ComputeProcessException
 from biolib.compute_node.utils import SystemExceptionCodes
-from biolib.typing_utils import List, Any, Dict, Optional
+from biolib.typing_utils import List, Dict, Optional
 from biolib.utils import get_absolute_container_image_uri
 
 
-class DockerExecutor(BaseExecutor):
+class DockerExecutor:
 
     def __init__(self, options: LocalExecutorOptions):
-        super().__init__(options)
+        self._options: LocalExecutorOptions = options
+        self._is_cleaning_up = False
 
         # TODO: Move this to shared class
         ##########
@@ -91,15 +94,18 @@ class DockerExecutor(BaseExecutor):
         self._tmp_secrets_dir = tempfile.TemporaryDirectory(dir=utils.BIOLIB_SECRETS_TMPFS_PATH or None)
         os.chmod(self._tmp_secrets_dir.name, 0o755)
 
-    def execute_module(self, module_input_serialized: bytes) -> bytes:
+    @property
+    def _container(self) -> Container:
+        if self._docker_container is None:
+            raise Exception('Docker container was None')
+        return self._docker_container
+
+    def execute_module(self, module_input_serialized: bytes, module_output_path: str) -> None:
         try:
             job_uuid = self._options['job']['public_id']
             send_status_update = self._options['send_status_update']
-            send_system_exception = self._options['send_system_exception']
-
             module_input = ModuleInput(module_input_serialized).deserialize()
 
-            # TODO: fix these status updates such that they also make sense for run-dev
             send_status_update(StatusUpdate(progress=55, log_message='Pulling images...'))
 
             self._pull()
@@ -109,33 +115,22 @@ class DockerExecutor(BaseExecutor):
 
             logger_no_user_data.debug(f'Starting execution of {job_uuid}.')
             try:
-                stdout, stderr, exit_code, mapped_output_files = self._execute_helper(module_input)
+                self._execute_helper(module_input, module_output_path)
             except docker.errors.NotFound as docker_error:
                 raise DockerContainerNotFoundDuringExecutionException from docker_error
             except Exception as exception:
                 raise ComputeProcessException(
                     exception,
                     SystemExceptionCodes.FAILED_TO_RUN_COMPUTE_CONTAINER.value,
-                    self._send_system_exception
+                    self._send_system_exception,
                 ) from exception
             logger_no_user_data.debug(f'Completed execution of {job_uuid}.')
-
-            try:
-                module_output_serialized = ModuleOutput().serialize(stdout, stderr, exit_code, mapped_output_files)
-                logger.debug(f'Compute time: {time.time() - start_time}')
-                return module_output_serialized
-
-            except Exception as exception:
-                raise ComputeProcessException(
-                    exception,
-                    SystemExceptionCodes.FAILED_TO_SERIALIZE_AND_SEND_MODULE_OUTPUT.value,
-                    send_system_exception
-                ) from exception
+            logger_no_user_data.debug(f'Compute time: {time.time() - start_time}')
         finally:
             try:
                 self.cleanup()
             except Exception:  # pylint: disable=broad-except
-                logger.error('DockerExecutor failed to clean up container')
+                logger_no_user_data.error('DockerExecutor failed to clean up container')
 
     def _pull(self):
         try:
@@ -153,7 +148,7 @@ class DockerExecutor(BaseExecutor):
                     docker_client.images.get(self._image_uri)
                 except ImageNotFound:
                     docker_client.images.pull(self._image_uri, auth_config=self._docker_auth_config)
-            logger.debug(f'Pulled image in: {time.time() - start_time}')
+            logger_no_user_data.debug(f'Pulled image in: {time.time() - start_time}')
 
         except Exception as exception:
             raise ComputeProcessException(
@@ -163,7 +158,7 @@ class DockerExecutor(BaseExecutor):
                 may_contain_user_data=False
             ) from exception
 
-    def _execute_helper(self, module_input):
+    def _execute_helper(self, module_input, module_output_path: str) -> None:
         job_uuid = self._options['job']['public_id']
         logger_no_user_data.debug(f'Initializing container for {job_uuid}.')
         self._initialize_docker_container(module_input)
@@ -178,13 +173,14 @@ class DockerExecutor(BaseExecutor):
         logger_no_user_data.debug(f'Starting Docker container for {job_uuid}')
 
         stdout_and_stderr_stream = docker_api_client.attach(
-            container=self._docker_container.id,
+            container=self._container.id,
             stderr=True,
             stdout=True,
             stream=True,
         )
 
-        self._docker_container.start()
+        pre_start_diff = self._container.diff()
+        self._container.start()
 
         if self._options['job']['app_version'].get('stdout_render_type') != 'markdown':
             for stdout_and_stderr in stdout_and_stderr_stream:
@@ -193,7 +189,7 @@ class DockerExecutor(BaseExecutor):
 
                 self._send_stdout_and_stderr(stdout_and_stderr)
 
-        exit_code = docker_api_client.wait(self._docker_container.id)['StatusCode']
+        exit_code = docker_api_client.wait(self._container.id)['StatusCode']
         # 137 is the error code from linux OOM killer (Should catch 90% of OOM errors)
         if exit_code == 137:
             raise ComputeProcessException(
@@ -204,12 +200,17 @@ class DockerExecutor(BaseExecutor):
 
         logger_no_user_data.debug(f'Docker container exited with code {exit_code} for {job_uuid}')
 
-        full_stdout = docker_api_client.logs(self._docker_container.id, stdout=True, stderr=False)
-        full_stderr = docker_api_client.logs(self._docker_container.id, stdout=False, stderr=True)
-
-        mapped_output_files = self._get_output_files(arguments=module_input['arguments'])
-        return full_stdout, full_stderr, exit_code, mapped_output_files
+        full_stdout = docker_api_client.logs(self._container.id, stdout=True, stderr=False)
+        full_stderr = docker_api_client.logs(self._container.id, stdout=False, stderr=True)
 
+        self._write_module_output_to_file(
+            arguments=module_input['arguments'],
+            exit_code=exit_code,
+            module_output_path=module_output_path,
+            stderr=full_stderr,
+            stdout=full_stdout,
+            pre_start_diff=pre_start_diff,
+        )
 
     def cleanup(self):
         # Don't clean up if already in the process of doing so, or done doing so
@@ -222,7 +223,7 @@ class DockerExecutor(BaseExecutor):
         for path_to_delete in [self._input_tar_path, self._runtime_tar_path]:
             if os.path.exists(path_to_delete):
                 os.remove(path_to_delete)
-        logger.debug(f"Deleted tars in: {time.time() - tar_time}")
+        logger_no_user_data.debug(f"Deleted tars in: {time.time() - tar_time}")
 
         container_time = time.time()
         if self._docker_container:
@@ -234,7 +235,7 @@ class DockerExecutor(BaseExecutor):
                 job_id=self._options['job']['public_id']
             )
 
-        logger.debug(f"Deleted compute container in: {time.time() - container_time}")
+        logger_no_user_data.debug(f"Deleted compute container in: {time.time() - container_time}")
         self._tmp_secrets_dir.cleanup()
 
     # TODO: type this method
@@ -247,7 +248,6 @@ class DockerExecutor(BaseExecutor):
 
             internal_network = self._options['internal_network']
             extra_hosts: Dict[str, str] = {}
-            dns_list: List[str] = []
 
             biolib_system_secret = {
                 'version': '1.0.0',
@@ -295,11 +295,6 @@ class DockerExecutor(BaseExecutor):
                 else:
                     extra_hosts[proxy.hostname] = proxy_ip
 
-            if self._options['dns_proxy']:
-                container_networks = self._options['dns_proxy'].attrs['NetworkSettings']['Networks']
-                dns_proxy_ip_address: str = container_networks[internal_network.name]['IPAddress']
-                dns_list = [dns_proxy_ip_address]
-
             create_container_args = {
                 'environment': environment_vars,
                 'extra_hosts': extra_hosts,
@@ -307,7 +302,6 @@ class DockerExecutor(BaseExecutor):
                 'mounts': docker_volume_mounts,
                 'network': internal_network.name,
                 'working_dir': module['working_directory'],
-                'dns': dns_list,
             }
 
             if self._options['job'].get('arguments_override_command'):
@@ -345,7 +339,7 @@ class DockerExecutor(BaseExecutor):
                 **create_container_args
             )
 
-            logger.debug('Finished initializing docker container')
+            logger_no_user_data.debug('Finished initializing docker container')
         except Exception as exception:
             raise ComputeProcessException(
                 exception,
@@ -435,85 +429,94 @@ class DockerExecutor(BaseExecutor):
                 self._send_system_exception
             ) from exception
 
-    def _get_output_files(self, arguments: List[str]):
-        module = self._options['module']
+    def _write_module_output_to_file(
+            self,
+            arguments: List[str],
+            exit_code: int,
+            module_output_path: str,
+            stderr: bytes,
+            stdout: bytes,
+            pre_start_diff: List[Dict],
+    ) -> None:
+        mapped_files: List[FileInContainer] = []
         try:
-            if self._docker_container is None:
-                raise Exception('Docker container was None')
+            mappings = Mappings(mappings_list=self._options['module']['output_files_mappings'], arguments=arguments)
+            changed_files: List[FileInContainer] = self._get_changed_files_in_docker_container(pre_start_diff)
 
-            docker_api_client = BiolibDockerClient.get_docker_client().api
-
-            # TODO: fix typing
-            input_tar: Any = None
-            if os.path.exists(self._input_tar_path):
-                input_tar = tarfile.open(self._input_tar_path)
-                input_tar_filelist = input_tar.getnames()
-
-            # TODO: fix typing
-            runtime_tar: Any = None
-            if os.path.exists(self._runtime_tar_path):
-                runtime_tar = tarfile.open(self._runtime_tar_path)
-                runtime_tar_filelist = runtime_tar.getnames()
-
-            mapped_output_files = {}
-            for mapping in module['output_files_mappings']:
-                try:
-                    tar_bytes_generator, _ = docker_api_client.get_archive(
-                        self._docker_container.id, mapping['from_path'])
-                except APIError:
-                    logger.warning(f'Could not get output from path {mapping["from_path"]}')
-                    continue
-
-                temp_tar_file = tempfile.NamedTemporaryFile(delete=False)
-                with temp_tar_file as tar_file:
-                    for chunk in tar_bytes_generator:
-                        tar_file.write(chunk)
-
-                tar = tarfile.open(temp_tar_file.name)
-
-                for file in tar.getmembers():
-                    file_obj = tar.extractfile(file)
-
-                    # Skip empty dirs
-                    if not file_obj:
-                        continue
-                    file_data = file_obj.read()
-
-                    # Remove parent dir from tar file name and prepend from_path.
-                    # Except if from_path is root '/', that works out of the box
-                    if mapping['from_path'].endswith('/') and mapping['from_path'] != '/':
-                        file_name = mapping['from_path'] + path_without_first_folder(file.name)
-
-                    # When getting a file use the from_path.
-                    # This is due to directory info (absolute path) being lost when using get_archive on files
-                    else:
-                        file_name = mapping['from_path']
-
-                    # Filter out unchanged input files
-                    if input_tar and file_name in input_tar_filelist and \
-                            input_tar.extractfile(file_name).read() == file_data:
-                        continue
-
-                    # Filter out unchanged source files if provided
-                    if runtime_tar and file_name in runtime_tar_filelist and runtime_tar.extractfile(
-                            file_name).read() == file_data:
-                        continue
-
-                    mapped_file_names = Mappings([mapping], arguments).get_mappings_for_path(file_name)
-                    for mapped_file_name in mapped_file_names:
-                        mapped_output_files[mapped_file_name] = file_data
-
-                tar.close()
-                os.unlink(temp_tar_file.name)
+            for file in changed_files:
+                if file.is_file():
+                    for mapped_path in mappings.get_mappings_for_path(file.path):
+                        mapped_output_file = copy(file)
+                        mapped_output_file.path = mapped_path
+                        mapped_files.append(mapped_output_file)
 
         except Exception as exception:
             raise ComputeProcessException(
                 exception,
                 SystemExceptionCodes.FAILED_TO_RETRIEVE_AND_MAP_OUTPUT_FILES.value,
-                self._send_system_exception
+                self._send_system_exception,
             ) from exception
 
-        return mapped_output_files
+        try:
+            ModuleOutputV2.write_to_file(
+                exit_code=exit_code,
+                files=mapped_files,
+                output_file_path=module_output_path,
+                stderr=stderr,
+                stdout=stdout,
+            )
+        except Exception as exception:
+            raise ComputeProcessException(
+                exception,
+                SystemExceptionCodes.FAILED_TO_SERIALIZE_AND_SEND_MODULE_OUTPUT.value,
+                self._send_system_exception,
+            ) from exception
+
+    def _get_changed_files_in_docker_container(self, pre_start_diff: List[Dict]) -> List[FileInContainer]:
+        from_mappings = [mapping['from_path'] for mapping in self._options['module']['output_files_mappings']]
+        pre_start_diff_paths = [obj['Path'] for obj in pre_start_diff]
+        post_run_diff = self._container.diff()
+        run_diff_paths: List[str] = [
+            obj['Path'] for obj in post_run_diff if
+            obj['Kind'] in (DockerDiffKind.CHANGED.value, DockerDiffKind.ADDED.value) and
+            obj['Path'] not in pre_start_diff_paths
+        ]
+
+        known_directories = set()
+        for path in run_diff_paths:
+            parent_folders = path.split('/')[:-1]
+            for idx in range(len(parent_folders)):
+                if idx == 0:
+                    continue  # always skip root
+
+                folder = '/' + '/'.join(parent_folders[1:idx + 1])
+                known_directories.add(folder)
+
+        def path_is_included_in_from_mappings(path: str) -> bool:
+            for from_mapping in from_mappings:
+                if path.startswith(from_mapping):
+                    return True
+            return False
+
+        overlay_upper_dir_path = self._container.attrs['GraphDriver']['Data']['UpperDir']
+        if not os.path.exists(overlay_upper_dir_path):
+            overlay_upper_dir_path = None
+            logger_no_user_data.debug(
+                'Docker Overlay2 UpperDir not available. Falling back to container.get_archive() for file extraction'
+            )
+
+        logger.debug(f'overlay_upper_dir_path={overlay_upper_dir_path}')
+
+        files_and_empty_dirs: List[FileInContainer] = []
+        for path in run_diff_paths:
+            if path not in known_directories and path_is_included_in_from_mappings(path):
+                files_and_empty_dirs.append(FileInContainer(
+                    container=self._container,
+                    overlay_upper_dir_path=overlay_upper_dir_path,
+                    path_in_container=path,
+                ))
+
+        return files_and_empty_dirs
 
     def _create_secrets_mount(self, secrets: Dict[str, str]) -> docker.types.Mount:
         job_uuid = self._options['job']['public_id']
