@@ -6,7 +6,8 @@ from biolib import utils
 from biolib.biolib_api_client import BiolibApiClient, CreatedJobDict
 from biolib.biolib_api_client.biolib_job_api import BiolibJobApi
 from biolib.biolib_binary_format.utils import RemoteIndexableBuffer
-from biolib.biolib_binary_format.unencrypted_module_output import UnencryptedModuleOutput
+from biolib.biolib_binary_format.remote_endpoints import RemoteJobStorageResultEndpoint
+from biolib.biolib_binary_format import ModuleOutputV2
 from biolib.compute_node.cloud_utils import CloudUtils
 from biolib.biolib_logging import logger_no_user_data
 from biolib.utils.multipart_uploader import get_chunk_iterator_from_file_object
@@ -14,7 +15,6 @@ from biolib.utils.multipart_uploader import get_chunk_iterator_from_file_object
 
 class JobStorage:
     module_output_file_name = 'module-output.bbf'
-    legacy_module_output_file_name = 'legacy-module-output.bbf'
 
     @staticmethod
     def upload_module_input(job: CreatedJobDict, module_input_serialized: bytes) -> None:
@@ -102,12 +102,10 @@ class JobStorage:
         return data
 
     @staticmethod
-    def get_module_output(job: CreatedJobDict) -> UnencryptedModuleOutput:
-        presigned_download_url = BiolibJobApi.get_job_storage_download_url(
-            job_uuid=job['public_id'],
-            job_auth_token=job['auth_token'],
-            storage_type='results',
+    def get_module_output(job: CreatedJobDict) -> ModuleOutputV2:
+        remote_job_storage_endpoint = RemoteJobStorageResultEndpoint(
+            job_id=job['public_id'],
+            job_auth_token=job['auth_token']
         )
-
-        buffer = RemoteIndexableBuffer(url=presigned_download_url)
-        return UnencryptedModuleOutput(buffer)
+        buffer = RemoteIndexableBuffer(endpoint=remote_job_storage_endpoint)
+        return ModuleOutputV2(buffer)
