@@ -2,9 +2,10 @@ import io
 import json
 import socket
 import shlex
-import tarfile
+import sys
+import tempfile
 import zipfile
-from time import time, sleep
+from time import time
 from queue import Queue
 import multiprocessing
 import os
@@ -12,9 +13,7 @@ import signal
 from types import FrameType
 
 import requests
-from docker.errors import ImageNotFound  # type: ignore
 from docker.models.networks import Network  # type: ignore
-from docker.models.containers import Container  # type: ignore
 
 from biolib.biolib_binary_format.stdout_and_stderr import StdoutAndStderr
 from biolib.compute_node.job_worker.job_legacy_input_wait_timeout_thread import JobLegacyInputWaitTimeout
@@ -32,7 +31,6 @@ from biolib.biolib_api_client.biolib_job_api import BiolibJobApi
 from biolib.biolib_docker_client import BiolibDockerClient
 from biolib.biolib_logging import logger, logger_no_user_data
 from biolib.compute_node.job_worker.executors import DockerExecutor
-from biolib.compute_node.job_worker.executors.base_executor import BaseExecutor
 from biolib.compute_node.job_worker.executors.types import LocalExecutorOptions, StatusUpdate
 from biolib.compute_node.socker_listener_thread import SocketListenerThread
 from biolib.compute_node.socket_sender_thread import SocketSenderThread
@@ -40,10 +38,8 @@ from biolib.compute_node.job_worker.mappings import Mappings, path_without_first
 from biolib.compute_node.job_worker.utils import ComputeProcessException, log_disk_and_memory_usage_info
 from biolib.compute_node.utils import get_package_type, SystemExceptionCodes, SystemExceptionCodeMap
 from biolib.biolib_binary_format import SavedJob, SystemStatusUpdate, ModuleInput, SystemException, \
-    UnencryptedModuleOutput
+    ModuleOutputV2, InMemoryIndexableBuffer
 
-_DNS_PROXY_IMAGE_URI = 'spx01/blocky@sha256:485ef739233341d3e14a64db646754aba7322c34645f804c3c0c00556d9a2cd1'
-DEFAULT_BUFFER_SIZE = 1024
 SOCKET_HOST = '127.0.0.1'
 
 
@@ -57,15 +53,16 @@ class JobWorkerProcess(multiprocessing.Process):
 
     # note: this method is run in the newly started process once called with .start()
     def run(self) -> None:
-        _JobWorker(self._socket_port, self._log_level).run_handle_message_loop()
+        JobWorker(self._socket_port, self._log_level).run_handle_message_loop()
 
 
-class _JobWorker:
+class JobWorker:
     _STOP_HANDLE_MESSAGE_LOOP = b'STOP_HANDLE_MESSAGE_LOOP'
 
-    def __init__(self, socket_port: int, log_level: int):
+    def __init__(self, socket_port: Optional[int] = None, log_level: Optional[int] = None):
         try:
-            logger.setLevel(log_level)
+            if log_level:
+                logger.setLevel(log_level)
 
             # handle interrupt from keyboard (CTRL + C)
             signal.signal(signal.SIGINT, self._handle_exit_gracefully)
@@ -81,12 +78,10 @@ class _JobWorker:
             self._jobs: Dict[str, CreatedJobDict] = {}
             self._root_job_wrapper: Optional[JobWrapper] = None
 
-            self._dns_proxy: Optional[Container] = None
-
             self._remote_host_proxies: List[RemoteHostProxy] = []
             self._internal_network: Optional[Network] = None
             self._public_network: Optional[Network] = None
-            self._executors: List[BaseExecutor] = []
+            self._executors: List[DockerExecutor] = []
             self.is_cleaning_up: bool = False
 
             self.job_temporary_dir: Optional[str] = None
@@ -99,7 +94,8 @@ class _JobWorker:
                 may_contain_user_data=False
             ) from exception
 
-        self._connect_to_parent()
+        if socket_port:
+            self._connect_to_parent()
 
     def _handle_exit_gracefully(self, signum: int, frame: FrameType) -> None:  # pylint: disable=unused-argument
         job_id = self._root_job_wrapper["job"]["public_id"] if self._root_job_wrapper else None
@@ -209,14 +205,6 @@ class _JobWorker:
             self._remote_host_proxies = []
             logger_no_user_data.debug(f'Cleaned up {proxy_count} proxies in {time() - proxy_cleanup_start_time}')
 
-        if self._dns_proxy:
-            try:
-                self._dns_proxy.remove(force=True)
-                logger_no_user_data.debug('Cleaned up DNS Proxy')
-            except Exception as exception:  # pylint: disable=broad-except
-                logger_no_user_data.error('Failed to clean up DNS Proxy')
-                logger.error(exception)
-
         self._cleanup_network(self._internal_network)
         self._internal_network = None
         self._cleanup_network(self._public_network)
@@ -324,11 +312,6 @@ class _JobWorker:
                     remote_host_proxy.start()
                     self._remote_host_proxies.append(remote_host_proxy)
 
-                if utils.BIOLIB_ENABLE_DNS_PROXY:
-                    domains_to_whitelist = list(hostname_to_ports.keys())
-                    logger_no_user_data.debug(f'Job "{job_id}" starting DNS Proxy for domains: {domains_to_whitelist}')
-                    self._start_dns_proxy(domains_to_whitelist=domains_to_whitelist)
-
             except Exception as exception:
                 raise ComputeProcessException(
                     exception,
@@ -339,81 +322,18 @@ class _JobWorker:
 
             logger_no_user_data.debug(f'Job "{job_id}" startup of remote host proxies completed')
 
-    def _get_dns_proxy_image(self):
-        docker = BiolibDockerClient.get_docker_client()
-        try:
-            return docker.images.get(_DNS_PROXY_IMAGE_URI)
-        except ImageNotFound:
-            logger_no_user_data.debug('Pulling DNS proxy Docker image...')
-            return docker.images.pull(_DNS_PROXY_IMAGE_URI)
-
-    def _start_dns_proxy(self, domains_to_whitelist):
-        docker = BiolibDockerClient.get_docker_client()
-        self._dns_proxy = docker.containers.create(
-            detach=True,
-            image=self._get_dns_proxy_image(),
-            name=f'biolib-dns-proxy-{self._root_job_wrapper["job"]["public_id"]}',
-            network=self._public_network.name,
-        )
-
-        blocky_config = '''
-upstream:
-    default:
-        - 127.0.0.11
-blocking:
-    whiteLists:
-        remoteHosts:
-            - |'''
-        for domain in domains_to_whitelist:
-            blocky_config += f'''
-              {domain}'''
-        blocky_config += '''
-    clientGroupsBlock:
-        default:
-            - remoteHosts
-port: 53
-'''
-
-        blocky_config_bytes = blocky_config.encode()
-        tarfile_in_memory = io.BytesIO()
-        with tarfile.open(fileobj=tarfile_in_memory, mode='w:gz') as tar:
-            info = tarfile.TarInfo('/config.yml')
-            info.size = len(blocky_config_bytes)
-            tar.addfile(info, io.BytesIO(blocky_config_bytes))
-
-        tarfile_bytes = tarfile_in_memory.getvalue()
-        tarfile_in_memory.close()
-        docker.api.put_archive(self._dns_proxy.id, '/app', tarfile_bytes)
-
-        self._internal_network.connect(self._dns_proxy.id)
-
-        self._dns_proxy.start()
-
-        proxy_is_ready = False
-        for retry_count in range(1, 5):
-            sleep(0.5 * retry_count)
-            # Use the container logs as a health check.
-            if b'TCP server is up and running' in self._dns_proxy.logs():
-                proxy_is_ready = True
-                break
-
-        if not proxy_is_ready:
-            self._dns_proxy.remove(force=True)
-            raise Exception('DNS Proxy did not start properly')
-
-        self._dns_proxy.reload()
-
     def _run_app_version(
-        self,
-        app_version_id: str,
-        module_input_serialized: bytes,
-        caller_job: CreatedJobDict
-    ) -> bytes:
+            self,
+            app_version_id: str,
+            module_input_serialized: bytes,
+            caller_job: CreatedJobDict,
+            main_module_output_path: str,
+    ) -> None:
         job: CreatedJobDict = BiolibJobApi.create(app_version_id, caller_job=caller_job['public_id'])
         self._jobs[job['public_id']] = job
-        return self._run_job(job, module_input_serialized)
+        self._run_job(job, module_input_serialized, main_module_output_path)
 
-    def _run_job(self, job: CreatedJobDict, module_input_serialized: bytes) -> bytes:
+    def _run_job(self, job: CreatedJobDict, module_input_serialized: bytes, main_module_output_path: str) -> None:
         job_uuid = job['public_id']
         logger_no_user_data.info(f'Job "{job_uuid}" running...')
         if self._root_job_wrapper is None:
@@ -450,7 +370,6 @@ port: 53
             for lfs_mapping in module['large_file_systems']:
                 logger_no_user_data.debug(f'Job "{job_uuid}" creating LFS for module "{module["name"]}"...')
                 lfs = LargeFileSystem(
-                    is_job_federated=job['federated_job_uuid'] is not None,
                     job_id=job['public_id'],
                     lfs_mapping=lfs_mapping,
                     send_status_update=self._send_status_update,
@@ -464,7 +383,7 @@ port: 53
         if source_files_are_mapped:
             runtime_zip_bytes = self._get_runtime_zip_as_bytes(root_job_id=root_job_id, app_version=job['app_version'])
 
-        module_output_serialized = self._run_module(
+        self._run_module(
             LocalExecutorOptions(
                 access_token=self._root_job_wrapper['access_token'],
                 biolib_base_url=self._root_job_wrapper['BASE_URL'],
@@ -476,29 +395,32 @@ port: 53
                 module=main_module,
                 remote_host_proxies=self._remote_host_proxies,
                 root_job_id=root_job_id,
-                dns_proxy=self._dns_proxy,
                 runtime_zip_bytes=runtime_zip_bytes,
                 send_status_update=self._send_status_update,
                 send_system_exception=self.send_system_exception,
                 send_stdout_and_stderr=self.send_stdout_and_stderr,
             ),
             module_input_serialized,
+            main_module_output_path,
         )
 
-        for lfs in lfs_dict.values():
-            lfs.detach()
-
         if utils.IS_RUNNING_IN_CLOUD:
             # Log memory and disk after pulling and executing module
             log_disk_and_memory_usage_info()
 
-        return module_output_serialized
-
-    def _run_module(self, options: LocalExecutorOptions, module_input_serialized: bytes) -> bytes:
+    def _run_module(
+            self,
+            options: LocalExecutorOptions,
+            module_input_serialized: bytes,
+            module_output_path: str,
+    ) -> None:
         module = options['module']
-        executor_instance: BaseExecutor
+        job_id = options['job']['public_id']
+        logger_no_user_data.debug(f'Job "{job_id}" running module "{module["name"]}"...')
+        executor_instance: DockerExecutor
 
         if module['environment'] == ModuleEnvironment.BIOLIB_APP.value:
+            logger_no_user_data.debug(f'Job "{job_id}" starting child job...')
             module_input = ModuleInput(module_input_serialized).deserialize()
             module_input_with_runtime_zip = self._add_runtime_zip_and_command_to_module_input(options, module_input)
             module_input_with_runtime_zip_serialized = ModuleInput().serialize(
@@ -506,13 +428,27 @@ port: 53
                 arguments=module_input_with_runtime_zip['arguments'],
                 files=module_input_with_runtime_zip['files'],
             )
-            return self._run_app_version(module['image_uri'], module_input_with_runtime_zip_serialized, options['job'])
+            return self._run_app_version(
+                module['image_uri'],
+                module_input_with_runtime_zip_serialized,
+                options['job'],
+                module_output_path,
+            )
 
         elif module['environment'] == ModuleEnvironment.BIOLIB_ECR.value and BiolibDockerClient.is_docker_running():
-            executor_instance = DockerExecutor(options)
-
+            try:
+                executor_instance = DockerExecutor(options)
+            except Exception as exception:
+                raise ComputeProcessException(
+                    exception,
+                    SystemExceptionCodes.FAILED_TO_INITIALIZE_DOCKER_EXECUTOR.value,
+                    self.send_system_exception,
+                    may_contain_user_data=False
+                ) from exception
         else:
-            raise Exception(f"Unsupported module environment {module['environment']}")
+            err_string = f'Job "{job_id}" hit unsupported module environment "{module["environment"]}"'
+            logger_no_user_data.error(err_string)
+            raise Exception(err_string)
 
         self._executors.append(executor_instance)
 
@@ -520,7 +456,7 @@ port: 53
             # Log memory and disk before pulling and executing module
             log_disk_and_memory_usage_info()
 
-        return executor_instance.execute_module(module_input_serialized)
+        executor_instance.execute_module(module_input_serialized, module_output_path)
 
     def _connect_to_parent(self):
         try:
@@ -626,12 +562,14 @@ port: 53
         system_exception_package = SystemException().serialize(biolib_exception_code)
         self._messages_to_send_queue.put(system_exception_package)
 
-    def send_stdout_and_stderr(self, stdout_and_stderr_bytes: bytes):
-        stdout_and_stderr_package = StdoutAndStderr().serialize(
-            stdout_and_stderr_bytes=stdout_and_stderr_bytes,
-        )
-
-        self._messages_to_send_queue.put(stdout_and_stderr_package)
+    def send_stdout_and_stderr(self, stdout_and_stderr_bytes: bytes) -> None:
+        if utils.IS_RUNNING_IN_CLOUD:
+            stdout_and_stderr_package = StdoutAndStderr().serialize(stdout_and_stderr_bytes=stdout_and_stderr_bytes)
+            self._messages_to_send_queue.put(stdout_and_stderr_package)
+        else:
+            sys.stdout.write(stdout_and_stderr_bytes.decode())
+            if not utils.IS_RUNNING_IN_NOTEBOOK:  # for some reason flushing in jupyter notebooks breaks \r handling
+                sys.stdout.flush()
 
     def _send_status_update(self, status_update: StatusUpdate) -> None:
         try:
@@ -649,35 +587,36 @@ port: 53
                 may_contain_user_data=False
             ) from exception
 
-    def _run_root_job(self, module_input_serialized: bytes) -> None:
+    def _run_root_job(self, module_input_serialized: bytes) -> str:
         # Make typechecker happy
         if not self._root_job_wrapper or not self.job_temporary_dir:
             raise BioLibError('Undefined job_wrapper or job_temporary_dir')
 
-        module_output_serialized = self._run_job(
+        main_module_output_path = os.path.join(self.job_temporary_dir, JobStorage.module_output_file_name)
+        self._run_job(
             job=self._root_job_wrapper['job'],
             module_input_serialized=module_input_serialized,
+            main_module_output_path=main_module_output_path,
         )
-
-        module_output_size = len(module_output_serialized)
-        job_uuid = self._root_job_wrapper['job']['public_id']
-        logger_no_user_data.debug(
-            f'Job "{job_uuid}" finished computation with module output size '
-            f'of {module_output_size} bytes'
-        )
-
-        # TODO: Remove when webserver endpoint /v1/job/<job_id>/result/ is deprecated
-        legacy_module_output_file_path = os.path.join(self.job_temporary_dir, JobStorage.legacy_module_output_file_name)
-        with open(legacy_module_output_file_path, mode='wb') as file:
-            file.write(module_output_serialized)
-
-        module_output_file_path = os.path.join(self.job_temporary_dir, JobStorage.module_output_file_name)
-        with open(legacy_module_output_file_path, 'rb') as legacy_module_output, \
-                open(module_output_file_path, 'wb') as module_output:
-
-            UnencryptedModuleOutput.write_to_file_from_legacy_module_output_file(
-                input_file=legacy_module_output,
-                output_file=module_output,
-            )
-
         self._send_status_update(StatusUpdate(progress=94, log_message='Computation finished'))
+        return main_module_output_path
+
+    def run_job_locally(self, job_dict: CreatedJobDict, module_input_serialized: bytes) -> ModuleOutputV2:
+        try:
+            with tempfile.TemporaryDirectory() as job_temporary_dir:
+                self.job_temporary_dir = job_temporary_dir
+                self._root_job_wrapper = JobWrapper(
+                    access_token=BiolibApiClient.get().access_token or '',
+                    BASE_URL=BiolibApiClient.get().base_url,
+                    cloud_job=None,
+                    compute_node_info=None,
+                    job=job_dict,
+                    job_temporary_dir=job_temporary_dir,
+                )
+                self._start_network_and_remote_host_proxies(job_dict)
+                module_output_path = self._run_root_job(module_input_serialized)
+                with open(module_output_path, mode='rb') as module_output_file:
+                    module_output_serialized = module_output_file.read()
+                return ModuleOutputV2(InMemoryIndexableBuffer(module_output_serialized))
+        finally:
+            self._cleanup()
