@@ -24,7 +24,7 @@ class RemoteHostExtended(RemoteHost):
 
 
 class RemoteHostProxy:
-    _DOCKER_IMAGE_URI = 'nginx:1.21.1-alpine'
+    _DOCKER_IMAGE_URI = 'public.ecr.aws/h5y4b3l1/biolib-remote-host-proxy:latest'
     _TRAFFIC_FORWARDER_PORT_OFFSET = 10000  # Port offset relative to port of a VSOCK proxy
 
     def __init__(
@@ -174,6 +174,7 @@ events {{
 }}
 
 http {{
+    client_max_body_size 0;
     map $request_method $bearer_token_on_post {{
         POST       "{bearer_token}";
         default    "";
@@ -189,6 +190,12 @@ http {{
         default    "";
     }}
 
+    map $request_method $bearer_token_on_patch_and_get {{
+        PATCH      "{bearer_token}";
+        GET        "{bearer_token}";
+        default    "";
+    }}
+
     server {{
         listen       80;
         resolver 127.0.0.11 valid=30s;
@@ -239,7 +246,17 @@ http {{
 
         location ~* "^/api/jobs/(?<job_id>[a-z0-9-]{{36}})/$" {{
             proxy_pass               {base_url}/api/jobs/$job_id/;
-            proxy_set_header         authorization $bearer_token_on_patch;
+            proxy_set_header         authorization $bearer_token_on_patch_and_get;
+            proxy_set_header         caller-job-uuid "{self._job_uuid}";
+            proxy_set_header         cookie "";
+            proxy_ssl_server_name    on;
+        }}
+
+        location ~* "^/api/jobs/create_job_with_data/$" {{
+            # Note: Using $1 here as URI part from regex must be used for proxy_pass
+            proxy_pass               {base_url}/api/jobs/create_job_with_data/$1;
+            proxy_set_header         authorization $bearer_token_on_post;
+            proxy_set_header         caller-job-uuid "{self._job_uuid}";
             proxy_set_header         cookie "";
             proxy_ssl_server_name    on;
         }}
@@ -248,6 +265,7 @@ http {{
             # Note: Using $1 here as URI part from regex must be used for proxy_pass
             proxy_pass               {base_url}/api/jobs/$1;
             proxy_set_header         authorization $bearer_token_on_post;
+            proxy_set_header         caller-job-uuid "{self._job_uuid}";
             proxy_set_header         cookie "";
             proxy_ssl_server_name    on;
         }}
