@@ -3,15 +3,18 @@ import collections.abc
 import multiprocessing
 import os
 import time
+import socket
 import sys
 from urllib.parse import urlparse
 
 import requests
 from importlib_metadata import version, PackageNotFoundError
 
+from biolib.utils.seq_util import SeqUtil, SeqUtilRecord
+
 # try fetching version, if it fails (usually when in dev), add default
 from biolib.biolib_errors import BioLibError
-from biolib.biolib_logging import logger_no_user_data
+from biolib.biolib_logging import logger_no_user_data, logger
 from biolib.typing_utils import Tuple, Iterator
 from .multipart_uploader import MultiPartUploader, get_chunk_iterator_from_bytes
 
@@ -22,7 +25,36 @@ except PackageNotFoundError:
 
 IS_DEV = os.getenv('BIOLIB_DEV', '').upper() == 'TRUE'
 
-BIOLIB_BASE_URL = os.getenv('BIOLIB_BASE_URL', default='https://biolib.com').lower()
+
+def _get_base_url() -> str:
+    base_url = os.getenv('BIOLIB_BASE_URL')
+    if base_url:
+        return base_url.lower().rstrip('/')
+
+    try:
+        search_list = []
+        with open('/etc/resolv.conf') as file:
+            for line in file:
+                line_trimmed = line.strip()
+                if line_trimmed.startswith('search'):
+                    search_list = line_trimmed.split()[1:]
+                    logger.debug(f'Found search list: {search_list} when resolving base url.')
+                    break
+
+        for search_host in search_list:
+            host_to_try = f'biolib.{search_host}'
+            try:
+                if len(socket.getaddrinfo(host_to_try, 443)) > 0:
+                    return f'https://{host_to_try}'.lower()
+            except BaseException:  # pylint: disable=broad-except
+                pass
+    except BaseException:  # pylint: disable=broad-except
+        pass
+
+    return 'https://biolib.com'
+
+
+BIOLIB_BASE_URL = _get_base_url()
 
 BIOLIB_CLOUD_BASE_URL = os.getenv('BIOLIB_CLOUD_BASE_URL', '').lower()
 
@@ -51,8 +83,6 @@ if BIOLIB_CLOUD_ENVIRONMENT and not IS_RUNNING_IN_CLOUD:
         'The compute node will not act as a cloud compute node'
     ))
 
-BIOLIB_ENABLE_DNS_PROXY = os.getenv('BIOLIB_ENABLE_DNS_PROXY', '').upper() == 'TRUE'
-
 RUN_DEV_JOB_ID = 'run-dev-mocked-job-id'
 
 
@@ -170,17 +200,3 @@ def download_presigned_s3_url(presigned_url: str, output_file_path: str) -> None
         logger_no_user_data.debug('Closing process poll...')
         process_pool.close()
         logger_no_user_data.debug('Process poll closed.')
-
-
-def stream_process_output(proc) -> None:
-    while proc.poll() is None:
-        stdout = io.TextIOWrapper(proc.stdout, newline='')
-        stderr = io.TextIOWrapper(proc.stderr, newline='')
-        try:
-            for line in stdout:
-                sys.stdout.write(line)
-
-            for line in stderr:
-                sys.stderr.write(line)
-        except Exception as error:  # pylint: disable=broad-except
-            logger_no_user_data.debug(f'Got an error while streaming process output: {error}')
