@@ -1,5 +1,6 @@
 import math
 import multiprocessing
+import multiprocessing.pool
 import os
 import time
 from urllib.parse import urlparse
@@ -55,10 +56,10 @@ class MultiPartUploader:
         self._complete_upload_request = complete_upload_request
         self._get_presigned_upload_url_request = get_presigned_upload_url_request
         self._start_multipart_upload_request = start_multipart_upload_request
+        self._bytes_uploaded: int = 0
 
     def upload(self, payload_iterator: Iterator[bytes], payload_size_in_bytes: int) -> None:
         parts: List[_PartMetadata] = []
-        bytes_uploaded: int = 0
 
         iterator_with_index: Iterator[_UploadChunkInputType] = enumerate(payload_iterator, 1)  # type: ignore
         logger_no_user_data.debug(f'Starting multipart upload of payload with size {payload_size_in_bytes} bytes')
@@ -88,19 +89,17 @@ class MultiPartUploader:
                 self._update_progress_bar_and_parts(
                     upload_chunk_response=upload_chunk_response,
                     parts=parts,
-                    bytes_uploaded=bytes_uploaded,
                     payload_size_in_bytes=payload_size_in_bytes
                 )
         else:
             # use 16 cores, unless less is available
-            process_pool = multiprocessing.Pool(processes=min(16, multiprocessing.cpu_count() - 1))
+            process_pool = multiprocessing.pool.ThreadPool(processes=min(16, multiprocessing.cpu_count() - 1))
             try:
                 response: _UploadChunkReturnType
                 for response in process_pool.imap(self._upload_chunk, iterator_with_index):
                     self._update_progress_bar_and_parts(
                         upload_chunk_response=response,
                         parts=parts,
-                        bytes_uploaded=bytes_uploaded,
                         payload_size_in_bytes=payload_size_in_bytes
                     )
             finally:
@@ -117,7 +116,7 @@ class MultiPartUploader:
                 complete_upload_response = requests.post(
                     auth=BearerAuth(BiolibApiClient.get().access_token) if requires_biolib_auth else None,
                     headers=self._complete_upload_request['headers'],
-                    json={'parts': parts, 'size_bytes': bytes_uploaded},
+                    json={'parts': parts, 'size_bytes': self._bytes_uploaded},
                     timeout=30,
                     url=self._complete_upload_request['url'],
                 )
@@ -145,7 +144,7 @@ class MultiPartUploader:
             if requires_biolib_auth:
                 BiolibApiClient.refresh_auth_token()
 
-            logger_no_user_data.debug(f'Uploading chunk {part_number} of length {len(chunk)}...')
+            logger_no_user_data.debug(f'Uploading part number {part_number} with size {len(chunk)} bytes...')
             try:
                 logger_no_user_data.debug(f'Getting upload URL for chunk {part_number}...')
                 get_url_response = requests.get(
@@ -194,18 +193,17 @@ class MultiPartUploader:
         self,
         upload_chunk_response: _UploadChunkReturnType,
         parts: List[_PartMetadata],
-        bytes_uploaded: int,
         payload_size_in_bytes: int,
     ) -> None:
         part_metadata, chunk_byte_length = upload_chunk_response
         part_number = part_metadata['PartNumber']
 
         parts.append(part_metadata)
-        bytes_uploaded += chunk_byte_length
+        self._bytes_uploaded += chunk_byte_length
 
-        approx_progress_percent = min(bytes_uploaded / (payload_size_in_bytes + 1) * 100, 100)
+        approx_progress_percent = min(self._bytes_uploaded / (payload_size_in_bytes + 1) * 100, 100)
         approx_rounded_progress = round(approx_progress_percent, 2)
         logger_no_user_data.debug(
-            f'Uploaded part {part_number} of {chunk_byte_length} bytes, '
+            f'Uploaded part number {part_number} with size {chunk_byte_length} bytes, '
             f'the approximate progress is {approx_rounded_progress}%'
         )
